package com.sun.media.sound;

import java.io.ByteArrayInputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import java.io.InputStream;
import java.io.File;
import java.io.IOException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.sound.sampled.AudioSystem;
import java.net.URL;

public abstract class AudioFloatInputStream
{
    public AudioFloatInputStream() {
    }
    
    public static AudioFloatInputStream getInputStream(URL url) throws UnsupportedAudioFileException, IOException {
        return new AudioFloatInputStream.DirectAudioFloatInputStream(AudioSystem.getAudioInputStream(url));
    }
    
    public static AudioFloatInputStream getInputStream(File file) throws UnsupportedAudioFileException, IOException {
        return new AudioFloatInputStream.DirectAudioFloatInputStream(AudioSystem.getAudioInputStream(file));
    }
    
    public static AudioFloatInputStream getInputStream(InputStream stream) throws UnsupportedAudioFileException, IOException {
        return new AudioFloatInputStream.DirectAudioFloatInputStream(AudioSystem.getAudioInputStream(stream));
    }
    
    public static AudioFloatInputStream getInputStream(AudioInputStream audioInputStream) {
        return new DirectAudioFloatInputStream(audioInputStream);
    }
    
    public static AudioFloatInputStream getInputStream(AudioFormat format, byte[] buf, int offset, int length) {
        AudioFloatConverter converter = AudioFloatConverter.getConverter(format);
        if (converter != null) {
            return new AudioFloatInputStream.BytaArrayAudioFloatInputStream(converter, buf, offset, length);
        }
        return getInputStream(new AudioInputStream(new ByteArrayInputStream(buf, offset, length), format, (format.getFrameSize() == -1) ? -1L : ((long)(length / format.getFrameSize()))));
    }
    
    public abstract AudioFormat getFormat();
    
    public abstract long getFrameLength();
    
    public abstract int read(float[] p0, int p1, int p2) throws IOException;
    
    public final int read(float[] array) throws IOException {
        return this.read(array, 0, array.length);
    }
    
    public final float read() throws IOException {
        float[] array = { 0.0f };
        int read = this.read(array, 0, 1);
        if (read == -1 || read == 0) {
            return 0.0f;
        }
        return array[0];
    }
    
    public abstract long skip(long p0) throws IOException;
    
    public abstract int available() throws IOException;
    
    public abstract void close() throws IOException;
    
    public abstract void mark(int p0);
    
    public abstract boolean markSupported();
    
    public abstract void reset() throws IOException;
    
    private static class BytaArrayAudioFloatInputStream extends AudioFloatInputStream
    {
        private int pos;
        private int markpos;
        private final AudioFloatConverter converter;
        private final AudioFormat format;
        private final byte[] buffer;
        private final int buffer_offset;
        private final int buffer_len;
        private final int framesize_pc;
        
        BytaArrayAudioFloatInputStream(AudioFloatConverter converter, byte[] buffer, int buffer_offset, int n) {
            this.pos = 0;
            this.markpos = 0;
            this.converter = converter;
            this.format = converter.getFormat();
            this.buffer = buffer;
            this.buffer_offset = buffer_offset;
            this.framesize_pc = this.format.getFrameSize() / this.format.getChannels();
            this.buffer_len = n / this.framesize_pc;
        }
        
        @Override
        public AudioFormat getFormat() {
            return this.format;
        }
        
        @Override
        public long getFrameLength() {
            return (long)this.buffer_len;
        }
        
        @Override
        public int read(float[] array, int n, int n2) throws IOException {
            if (array == null) {
                throw new NullPointerException();
            }
            if (n < 0 || n2 < 0 || n2 > array.length - n) {
                throw new IndexOutOfBoundsException();
            }
            if (this.pos >= this.buffer_len) {
                return -1;
            }
            if (n2 == 0) {
                return 0;
            }
            if (this.pos + n2 > this.buffer_len) {
                n2 = this.buffer_len - this.pos;
            }
            this.converter.toFloatArray(this.buffer, this.buffer_offset + this.pos * this.framesize_pc, array, n, n2);
            this.pos += n2;
            return n2;
        }
        
        @Override
        public long skip(long n) throws IOException {
            if (this.pos >= this.buffer_len) {
                return -1L;
            }
            if (n <= 0L) {
                return 0L;
            }
            if ((long)this.pos + n > (long)this.buffer_len) {
                n = (long)(this.buffer_len - this.pos);
            }
            this.pos = (int)((long)this.pos + n);
            return n;
        }
        
        @Override
        public int available() throws IOException {
            return this.buffer_len - this.pos;
        }
        
        @Override
        public void close() throws IOException {
        }
        
        @Override
        public void mark(int n) {
            this.markpos = this.pos;
        }
        
        @Override
        public boolean markSupported() {
            return true;
        }
        
        @Override
        public void reset() throws IOException {
            this.pos = this.markpos;
        }
    }
    
    private static class DirectAudioFloatInputStream extends AudioFloatInputStream
    {
        private final AudioInputStream stream;
        private AudioFloatConverter converter;
        private final int framesize_pc;
        private byte[] buffer;
        
        DirectAudioFloatInputStream(AudioInputStream audioInputStream) {
            this.converter = AudioFloatConverter.getConverter(audioInputStream.getFormat());
            if (this.converter == null) {
                AudioFormat format = audioInputStream.getFormat();
                AudioFormat[] targetFormats = AudioSystem.getTargetFormats(AudioFormat.Encoding.PCM_SIGNED, format);
                AudioFormat targetFormat;
                if (targetFormats.length != 0) {
                    targetFormat = targetFormats[0];
                }
                else {
                    float sampleRate = format.getSampleRate();
                    format.getSampleSizeInBits();
                    format.getFrameSize();
                    format.getFrameRate();
                    int sampleSizeInBits = 16;
                    int frameSize = format.getChannels() * (sampleSizeInBits / 8);
                    targetFormat = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, sampleRate, sampleSizeInBits, format.getChannels(), frameSize, sampleRate, false);
                }
                audioInputStream = AudioSystem.getAudioInputStream(targetFormat, audioInputStream);
                this.converter = AudioFloatConverter.getConverter(audioInputStream.getFormat());
            }
            this.framesize_pc = audioInputStream.getFormat().getFrameSize() / audioInputStream.getFormat().getChannels();
            this.stream = audioInputStream;
        }
        
        @Override
        public AudioFormat getFormat() {
            return this.stream.getFormat();
        }
        
        @Override
        public long getFrameLength() {
            return this.stream.getFrameLength();
        }
        
        @Override
        public int read(float[] array, int n, int n2) throws IOException {
            int len = n2 * this.framesize_pc;
            if (this.buffer == null || this.buffer.length < len) {
                this.buffer = new byte[len];
            }
            int read = this.stream.read(this.buffer, 0, len);
            if (read == -1) {
                return -1;
            }
            this.converter.toFloatArray(this.buffer, array, n, read / this.framesize_pc);
            return read / this.framesize_pc;
        }
        
        @Override
        public long skip(long n) throws IOException {
            long skip = this.stream.skip(n * (long)this.framesize_pc);
            if (skip == -1L) {
                return -1L;
            }
            return skip / (long)this.framesize_pc;
        }
        
        @Override
        public int available() throws IOException {
            return this.stream.available() / this.framesize_pc;
        }
        
        @Override
        public void close() throws IOException {
            this.stream.close();
        }
        
        @Override
        public void mark(int n) {
            this.stream.mark(n * this.framesize_pc);
        }
        
        @Override
        public boolean markSupported() {
            return this.stream.markSupported();
        }
        
        @Override
        public void reset() throws IOException {
            this.stream.reset();
        }
    }
}
